// DeleteComponent.java
// $Id: DeleteComponent.java,v 1.8 2000/08/17 15:07:27 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html

package org.w3c.jwput.gui;

import java.util.Hashtable;

import java.awt.BorderLayout;
import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import javax.swing.BorderFactory;

import org.w3c.tools.widgets.Utilities;

import org.w3c.jwput.JWPut;
import org.w3c.jwput.JWUI;

/**
 * A JTabbedPane allowing user to delete a file on the web
 * @version $Revision: 1.8 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class DeleteComponent extends JTabbedPane implements HistoryListener {

    class Deleter extends Thread {
	String url;
	JWPut jwput;

	public void run() {
	    jwput.delete(url);
	}
	
	Deleter(JWPut jwput, String url) {
	    this.jwput = jwput;
	    this.url   = url;
	}
    }

    protected History urlhistory  = null;

    protected JComboBox dest   = null;

    protected JWPut jwput = null;

    protected DeleteComponent(Hashtable props) {
	super();
	initialize(props);
	build(props);
    }

    protected void initialize(Hashtable props) {
	this.jwput       = (JWPut) props.get(JWGUI.JWPUT_P);
	this.urlhistory  = (History) props.get(JWGUI.URL_HISTORY_P);
	urlhistory.addHistoryListener(this);
    }

    protected void build(Hashtable props) {
	JPanel panel = new JPanel(new BorderLayout());
	
	GridBagLayout gbl = new GridBagLayout();
 	GridBagConstraints gbc = new GridBagConstraints();
 	gbc.fill = GridBagConstraints.HORIZONTAL;
 	gbc.weightx = 0.9;
 	gbc.weighty = 0;
 	gbc.insets = Utilities.insets4;

	//
	// url to delete
	//
	dest = new JComboBox();
	urlhistory.initialize(dest);
	dest.setEditable(true);
	dest.setMaximumRowCount(5);

	JPanel p1 = new JPanel(new BorderLayout(4, 0));
	p1.setBorder(JWBorder.createTitledBorder("Document URL"));
	p1.add(dest, "Center");

 	GridBagLayout gbl2 = new GridBagLayout();
	JPanel destp = new JPanel(gbl2);

	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbl2.setConstraints(p1, gbc);
	destp.add(p1);

	//
	// delete
	//
	//JButton delete = new JButton(" Delete Document ");
	JButton delete = Icons.getIconButton("delete_button_normal.gif",
					     "delete_button_pressed.gif",
					     null);
	ActionListener al = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		delete();
	    }
	};
	delete.addActionListener(al);
	JPanel deletep = new JPanel();
	deletep.add(delete, "Center");
	deletep.setBorder(BorderFactory.createEmptyBorder(7, 7, 14, 7));

	//
	// source + destination + save
	//
	JPanel p3 = new JPanel(new GridLayout(1, 1));
	p3.add(destp);

	JPanel p4 = new JPanel(new BorderLayout());
	p4.setBorder(BorderFactory.createEmptyBorder(14, 7, 7, 7));
	p4.add(p3, "Center");

	panel.add(p4, "North");
	panel.add(deletep, "South");

	//
	// add to itself
	//
	add("Delete Document", panel);
    }

    public void ItemRemoved(HistoryEvent e) {
	History src = e.getSource();
	String  item   = e.getItem();
	if (src == urlhistory) {
	    dest.removeItem(item);
	}
    }

    public void ItemAdded(HistoryEvent e) {
	History src = e.getSource();
	String  item   = e.getItem();
	if (src == urlhistory) {
	    dest.insertItemAt(item, 0);
	    dest.setSelectedItem(item);
	}
    }

    public void ItemMovedFirst(HistoryEvent e) {
	History src  = e.getSource();
	String  item = e.getItem();
	if (src == urlhistory) {
	    dest.setSelectedItem(item);
	    int idx = dest.getSelectedIndex();
	    dest.insertItemAt(item, 0);
	    if (idx != -1) {
		dest.removeItemAt(idx+1);
	    }
	    dest.setSelectedIndex(0);
	}
    }

    protected void delete() {
	String url = (String)dest.getSelectedItem();
	urlhistory.add(url);
	//
	// perform the delete
	//
	(new Deleter(jwput, url)).start();
    }

}
