// LoadComponent.java
// $Id: LoadComponent.java,v 1.17 2000/08/17 15:07:27 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html

package org.w3c.jwput.gui;

import java.util.Hashtable;
import java.util.Properties;

import java.io.File;
import java.io.IOException;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;

import org.w3c.tools.widgets.Utilities;

import org.w3c.jwput.JWGet;
import org.w3c.jwput.JWUI;
import org.w3c.jwput.event.JWListener;
import org.w3c.jwput.event.JWEvent;

/**
 * A JTabbedPane allowing user to get a file from the web
 * @version $Revision: 1.17 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class LoadComponent extends JTabbedPane {

    class Loader extends Thread {
	File file;
	String url;
	JWGet jwget;
	Hashtable headers;

	public void run() {
	    try {
		enableAbort();
		setCursor(Cursor.WAIT_CURSOR);
		jwget.getFile(file, url, headers);
		setCursor(Cursor.DEFAULT_CURSOR);
		disableAbort();
	    } catch (IOException ex) {
		setCursor(Cursor.DEFAULT_CURSOR);
		disableAbort();
		ui.displayError(ex.getMessage(), "IO Error");
	    }
	}
	
	Loader(JWGet jwget, File file, String url, Hashtable headers) 
	{
	    this.jwget   = jwget;
	    this.file    = file;
	    this.url     = url;
	    this.headers = headers;
	}
    }

    class HL implements HistoryListener {

	public void ItemRemoved(HistoryEvent e) {
	    History src  = e.getSource();
	    String  item = e.getItem();
	    if (src == urlhistory) {
		source.removeItem(item);
	    } else if (src == filehistory) {
		dest.removeItem(item);
	    }
	}

	public void ItemMovedFirst(HistoryEvent e) {
	    History src  = e.getSource();
	    String  item = e.getItem();
	    JComboBox combo = null;
	    if (src == urlhistory) {
		combo = source;
	    } else if (src == filehistory) {
		combo = dest;
	    }
	    combo.setSelectedItem(item);
	    int idx = combo.getSelectedIndex();
	    combo.insertItemAt(item, 0);
	    if (idx != -1) {
		combo.removeItemAt(idx+1);
	    }
	    combo.setSelectedIndex(0);
	}

	public void ItemAdded(HistoryEvent e) {
	    History src  = e.getSource();
	    String  item = e.getItem();
	    if (src == urlhistory) {
		source.insertItemAt(item, 0);
		source.setSelectedItem(item);
	    } else if (src == filehistory) {
		dest.insertItemAt(item, 0);
		dest.setSelectedItem(item);
	    }
	}
    }

    //
    // listener
    //
    ItemListener itemlistener = new ItemListener() {
	public void itemStateChanged(ItemEvent ev) {
	    if (ev.getStateChange() == ItemEvent.SELECTED) {
		String url = (String)source.getSelectedItem();
		String file = synchfileurl.getFile(url);
		if (file != null) {
		    dest.removeItem(file);
		    dest.addItem(file);
		    dest.setSelectedItem(file);
		}
	    }
	}
    };

    JWListener jwlistener = new JWListener() {
	public void notify(JWEvent ev) {
	    String url  = ev.getURL();
	    String file = ev.getFile();
	    if (ev.getType() == ev.DOWNLOAD) {
		synchfileurl.put(url, file);
		urlhistory.add(url);
		filehistory.add(file);
	    }
	}
    };

    protected History urlhistory  = null;
    protected History filehistory = null;

    protected SynchFileURL synchfileurl = null;

    protected JComboBox source = null;
    protected JComboBox dest   = null;

    protected JWGet jwget = null;
    protected JWGUI ui    = null;

    protected ButtonGroup cacheOptions = null;
    protected JTextField caccept = null;

    protected JButton abortButton = null;

    protected LoadComponent(Hashtable props) {
	super();
	initialize(props);
	build(props);
    }

    protected void initialize(Hashtable props) {
	this.jwget        = (JWGet) props.get(JWGUI.JWGET_P);
	this.urlhistory   = (History) props.get(JWGUI.URL_HISTORY_P);
	this.filehistory  = (History) props.get(JWGUI.FILE_HISTORY_P);
	this.synchfileurl = (SynchFileURL) props.get(JWGUI.SYNCH_FILE_URL_P);
	this.ui           = (JWGUI) props.get(JWGUI.JWUI_P);
	HL hl = new HL();
	jwget.addJWListener(jwlistener);
	urlhistory.addHistoryListener(hl);
	filehistory.addHistoryListener(hl);
    }

    protected void build(Hashtable props) {
	JPanel panel = new JPanel(new BorderLayout());
	
	GridBagLayout gbl = new GridBagLayout();
 	GridBagConstraints gbc = new GridBagConstraints();
 	gbc.fill = GridBagConstraints.HORIZONTAL;
 	gbc.weightx = 0.9;
 	gbc.weighty = 0;
 	gbc.insets = Utilities.insets4;

	//
	// destination
	//
	dest = new JComboBox();
	filehistory.initialize(dest);
	dest.setEditable(true);
	dest.setMaximumRowCount(5);

	JButton browse = new JButton("Browse");
	browse.setActionCommand("select");
	browse.addActionListener(new BrowseListener(filehistory,
						    ui.getUserProperties(),
						    JWGUI.LOAD_DIR_P));

	JPanel p1 = new JPanel(new BorderLayout(4, 0));
	p1.setBorder(JWBorder.createTitledBorder("Destination"));
	p1.add(dest, "Center");
	p1.add(browse, "East");

	JPanel destp = new JPanel(gbl);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(p1, gbc);
	destp.add(p1);

	//
	// source
	//
	source = new JComboBox();
	urlhistory.initialize(source);
	source.setEditable(true);
	source.setMaximumRowCount(5);
	source.addItemListener(itemlistener);

	p1 = new JPanel(new BorderLayout(4, 0));
	p1.setBorder(JWBorder.createTitledBorder("Source"));
	p1.add(source, "Center");

 	GridBagLayout gbl2 = new GridBagLayout();
	JPanel sourcep = new JPanel(gbl2);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbl2.setConstraints(p1, gbc);
	sourcep.add(p1);

	//
	// save
	//
	//JButton load = new JButton(" Download Document ");
	JButton load = Icons.getIconButton("download_button_normal.gif",
					   "download_button_pressed.gif",
					   null);
	ActionListener al = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		load();
	    }
	};
	load.addActionListener(al);

	//
	// abort
	//
	abortButton = Icons.getIconButton("abort_button_normal.gif",
					  "abort_button_pressed.gif",
					  "abort_button_disabled.gif");
	ActionListener al2 = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		jwget.abort();
		setCursor(Cursor.DEFAULT_CURSOR);
		disableAbort();
	    }
	};
	abortButton.addActionListener(al2);
	abortButton.setEnabled(false);

	JPanel loadp = new JPanel();
	loadp.add(load, "West");
	loadp.add(abortButton, "East");
	loadp.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));

	//
	// source + destination + save
	//
	JPanel p3 = new JPanel(new GridLayout(2, 1));
	p3.add(sourcep);
	p3.add(destp);

	JPanel p4 = new JPanel(new BorderLayout());
	p4.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));
	p4.add(p3, "Center");

	panel.add(p4, "North");
	panel.add(loadp, "South");

	//
	// Options
	//

	JRadioButton r1 = new JRadioButton("Get nearest fresh version");
	r1.setActionCommand("r1");
	r1.setSelected(true);
	
	JRadioButton r2 = 
	    new JRadioButton("Force validation at nearest remote cache");
	r2.setActionCommand("r2");

	JRadioButton r3 = 
	    new JRadioButton("Force validation at origin server");
	r3.setActionCommand("r3");

	JRadioButton r4 = new JRadioButton("Force full reload");
	r4.setActionCommand("r4");
	
	// Group the radio buttons.
	cacheOptions = new ButtonGroup();
	cacheOptions.add(r1);
	cacheOptions.add(r2);
	cacheOptions.add(r3);
	cacheOptions.add(r4);

	JPanel pb = new JPanel(new GridLayout(4,1));
	pb.setBorder(JWBorder.createTitledBorder(7,7,7,7,"Cache Options"));
	pb.add(r1);
	pb.add(r2);
	pb.add(r3);
	pb.add(r4);

	JLabel acceptl = new JLabel("Accept", JLabel.LEFT);
	caccept = new JTextField(10);
	caccept.setBorder(BorderFactory.createLoweredBevelBorder());
	caccept.setEditable(true);

	JPanel pa = new JPanel();
	pa.setBorder(JWBorder.createTitledBorder(7,7,7,7,"HTTP Headers"));
	pa.add(acceptl);
	pa.add(caccept);

	JPanel prp = new JPanel(new GridLayout(1,2));
	prp.add(pb);
	prp.add(pa);

	JPanel pb2 = new JPanel(new BorderLayout());
	pb2.add(prp, "West");

	JPanel optionsp = new JPanel(new BorderLayout());
	optionsp.add(pb2, "North");

	//
	// add to itself
	//
	add("Download Document", panel);
	add("Request Properties", optionsp);
    }

    protected void load() {
	Hashtable headers = new Hashtable(5);

	String file = (String)dest.getSelectedItem();
	if (file == null)
	    file = ".";
	filehistory.add(file);
	String url = (String)source.getSelectedItem();
	if (url == null)
	    return;
	urlhistory.add(url);

	//
	// Cache Options
	//
	File   toload = new File(file);
	String cache  = cacheOptions.getSelection().getActionCommand();
	if (cache.equals("r1")) {
	    headers.put("Cache-Control", "proxy-revalidate");
	} else if (cache.equals("r2")) {
	    //FIXME file
	    String etag = jwget.getETag(toload, url);
	    if (etag != null)
		headers.put("If-None-Match", etag);
	} else if (cache.equals("r3")) {
	    String etag = jwget.getETag(toload, url);
	    if (etag != null)
		headers.put("If-None-Match", etag);
	    headers.put("Cache-Control", "no-cache");
	} else if (cache.equals("r4")) {
	    headers.put("Cache-Control", "no-cache, no-transform");
	}
	
	//
	// HTTP Headers
	//
	String accept = caccept.getText();
	if ((accept != null) && (! accept.equals(""))) {
	    headers.put("Accept", accept);
	}

	// check the directory...
	File dir = null;
	File toget = new File(file);
	if (file.endsWith(File.separator)) {
	    dir = toget;
	} else {
	    dir = toget.getParentFile();
	}
	if (! dir.exists()) {
	    String msg = "Directory \""+dir.getAbsolutePath()+
		"\" doesn't exists.";
	    String question = "Create it?";
	    if (ui.askQuestion(msg, question)) {
		if (! dir.mkdirs()) {
		    ui.displayError("Unable to create the directory.");
		    return;
		}
	    } else {
		return;
	    }
	}
	//
	// perform the get
	//
	(new Loader(jwget, toget, url, headers)).start();
    }

    protected void enableAbort() {
	abortButton.setEnabled(true);
    }

    protected void disableAbort() {
	abortButton.setEnabled(false);
    }

    protected void setCursor(int cursor) {
	Frame frame = 
	    (Frame)SwingUtilities.getAncestorOfClass(Frame.class, this);
	frame.setCursor(Cursor.getPredefinedCursor(cursor));
    }

}
