// SaveComponent.java
// $Id: SaveComponent.java,v 1.35 2000/08/17 15:07:28 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html

package org.w3c.jwput.gui;

import java.io.File;
import java.io.FileNotFoundException;

import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.TimeZone;
import java.util.Vector;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

import org.w3c.tools.widgets.Utilities;

import org.w3c.jwput.JW;
import org.w3c.jwput.JWPut;
import org.w3c.jwput.PutAbortedException;

import org.w3c.jwput.event.JWListener;
import org.w3c.jwput.event.JWEvent;
import org.w3c.www.mime.Utils;
/**
 * A JTabbedPane allowing user to put a file on the web
 * @version $Revision: 1.35 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class SaveComponent extends JTabbedPane {

    protected static String languages[] = { 
	"en",
	"fr",
	"ge",
	"jp" 
    };

    protected static String charsets[] = { 
	" ",
	"iso-2022-jp",
	"iso-8859-1",
	"iso-8859-2",
	"iso-8859-3",
	"iso-8859-4" 
    };

    protected static String encodings[] = { 
	" ",
	"compress",
	"deflate",
	"gzip" 
    };

    protected History urlhistory  = null;
    protected History filehistory = null;

    protected SynchFileURL synchfileurl = null;

    protected JComboBox source = null;
    protected JComboBox dest   = null;

    protected JComboBox  cmedia    = null;
    protected JComboBox  cencoding = null;
    protected JComboBox  ccharset  = null;
    protected JTextField clength   = null;
    protected JTextField clast     = null;
    protected JTextField centity   = null;

    protected JWPut jwput = null;
    protected JWGUI ui    = null;

    protected SimpleDateFormat formatter = null;

    protected SU sourceInfoUpdater = null;

    protected JButton abortButton = null;

    //
    // Inner Classes
    //

    class Saver extends Thread {
	
	File files[];
	String url;
	JWPut jwput;
	Hashtable headers;

 	public void run() {
	    setCursor(Cursor.WAIT_CURSOR);
	    int len = files.length;
	    for (int i = 0 ; i < len ; i++) {
		File file = files[i];
		try {
		    if (file.isDirectory()) {
			ui.hidePopups();
			String message[] = { 
			    "You are going to put an entire directory,",
			    "all files located in this directory will be",
			    "uploaded" 
			};
			String question[] = {
			    "Do you want to perform this action recursively?",
			    "(hit cancel to abort)."
			};
			int returnval = ui.ask(message, question);
			switch (returnval) 
			    {
			    case JWDialog.APPROVE_OPTION:
				enableAbort();
				jwput.putDir(file, url, headers, true);
				disableAbort();
				ui.showPopups();
				break;
			    case JWDialog.CANCEL_OPTION:
				enableAbort();
				jwput.putDir(file, url, headers, false);
				disableAbort();
				ui.showPopups();
				break;
			    case JWDialog.ABORT_OPTION:
				setCursor(Cursor.DEFAULT_CURSOR);
				ui.showPopups();
				return;
			    }
		    } else {
			ui.showPopups();
			enableAbort();
			jwput.putFile(file, url, headers);
			disableAbort();
		    }
		} catch (FileNotFoundException ex) {
		    ui.showPopups();
		    ui.displayError(ex.getMessage(), "File not found");
		}
	    }
	    setCursor(Cursor.DEFAULT_CURSOR);
	}
	
	Saver(JWPut jwput, File file, String url, Hashtable headers) {
	    super("HTTP Saver");
	    this.jwput   = jwput;
	    this.files   = new File[1];
	    this.url     = url;
	    this.headers = headers;
	    files[0] = file;
	}

	Saver(JWPut jwput, File[] files, String url, Hashtable headers) {
	    super("HTTP Saver");
	    this.jwput   = jwput;
	    this.files   = files;
	    this.url     = url;
	    this.headers = headers;
	}
    }

    class HL implements HistoryListener {

	public void ItemRemoved(HistoryEvent e) {
	    History src  = e.getSource();
	    String  item = e.getItem();
	    if (src == urlhistory) {
		dest.removeItem(item);
	    } else if (src == filehistory) {
		source.removeItem(item);
	    }
	}

	public void ItemMovedFirst(HistoryEvent e) {
	    History src  = e.getSource();
	    String  item = e.getItem();
	    JComboBox combo = null;
	    if (src == urlhistory) {
		combo = dest;
	    } else if (src == filehistory) {
		combo = source;
	    }
	    combo.setSelectedItem(item);
	    int idx = combo.getSelectedIndex();
	    combo.insertItemAt(item, 0);
	    if (idx != -1) {
		combo.removeItemAt(idx+1);
	    }
	    combo.setSelectedIndex(0);
	}

	public void ItemAdded(HistoryEvent e) {
	    History src  = e.getSource();
	    String  item = e.getItem();
	    if (src == urlhistory) {
		dest.insertItemAt(item, 0);
		dest.setSelectedItem(item);
	    } else if (src == filehistory) {
		source.insertItemAt(item, 0);
		source.setSelectedItem(item);
	    }
	}
    }

    class MediaTypeObserver implements ItemListener {
	
	JComboBox combo      = null;
	String    deselected = null;

	public void itemStateChanged(ItemEvent e) {
	    int state = e.getStateChange();
	    if (state == ItemEvent.DESELECTED) {
		deselected = (String) e.getItem();
	    } else { // SELECTED
		String selected = (String) e.getItem();
		String usual    = getUsualMime();
		if ((deselected != null)
		    && (usual != null)
		    && (! usual.equals("content/unknown"))
		    && (! selected.equalsIgnoreCase(usual))) {
		    // ask
		    String message[] = {
			"Warning!",
			selected+" is not the usual",
			"mime type for this kind of file."
		    };
		    String question = 
			"Do you really want to set this mime type?";
		    if (! ui.askQuestion(message, question)) {
			combo.setSelectedItem(deselected);
		    }
		}
	    }
	}

	String getUsualMime() {
	    String toput = getFile();
	    return (toput != null)?
		Utils.guessContentTypeFromName(toput)
		: null;
	}

	MediaTypeObserver(JComboBox combo) {
	    this.combo = combo;
	}

    }

    /**
     * Update Source Information
     */
    class SU implements ChangeListener {
	
	JComponent optionsp    = null;
	String     currentfile = null;
	
	public String getUserSelectedMime() {
	    String toput = getFile();
	    if ((toput != null) && (! toput.equals(currentfile))) {
		String mime = null;
		try {
		    mime = getJWPut().getMimeType(toput, false);
		} catch (PutAbortedException ex) {
		    mime = null;
		}
		if (mime == null)
		    mime = jwput.DEFAULT_MIME;
		//update source information 
		//FIXME bug in TextField of JComboBox
		cmedia.setSelectedItem(mime);
		return mime;
	    }
	    return (String)cmedia.getEditor().getItem();
	}

	/**
	 * ChangeListener implementation.
	 * @param e a ChangeEvent
	 */
	public void stateChanged(ChangeEvent e) {
	    Object source = e.getSource();
	    if (source instanceof JTabbedPane) {
		JTabbedPane pane = (JTabbedPane) source;
		if (pane.getSelectedComponent() == optionsp) {
		    String mime  = getUserSelectedMime();
		    String toput = getFile();
		    File file    = new File(toput);
		    if (! file.isDirectory()) {
			String length = String.valueOf(file.length());
			clength.setText(length);
		    } else {
			clength.setText("");
		    }
		    String modified = 
			formatter.format(new Date(file.lastModified()));
		    clast.setText(modified);
		    String url = getURL();
		    if (url != null) {
			String etag = getJWPut().getETag(file, url);
			if (etag != null)
			    centity.setText(etag);
			else
			    centity.setText("");
		    }
		    currentfile = toput;
		}
	    }
	}

	SU (JComponent optionsp) {
	    this.optionsp = optionsp;
	}
    }

    //
    // listener
    //

    ItemListener itemlistener = new ItemListener() {
	public void itemStateChanged(ItemEvent ev) {
	    if (ev.getStateChange() == ItemEvent.SELECTED) {
		String file = (String)source.getSelectedItem();
		String url = synchfileurl.getURL(file);
		if (url != null) {
		    if (! SwingUtils.isInComboBox(dest, url)) {
			dest.addItem(url);
		    }
		    dest.setSelectedItem(url);
		}
	    }
	}
    };

    JWListener jwlistener = new JWListener() {
	public void notify(JWEvent ev) {
	    String url  = ev.getURL();
	    String file = ev.getFile();
	    if (ev.getType() == ev.UPLOAD) {
		synchfileurl.put(url, file);
		urlhistory.add(url);
		filehistory.add(file);
	    }
	}
    };

    //
    // SaveComponent
    //

    protected SaveComponent(Hashtable props) {
	super();
	initialize(props);
	build(props);
    }

    protected void initialize(Hashtable props) {
	this.urlhistory  = (History) props.get(JWGUI.URL_HISTORY_P);
	this.filehistory = (History) props.get(JWGUI.FILE_HISTORY_P);
	this.synchfileurl = (SynchFileURL) props.get(JWGUI.SYNCH_FILE_URL_P);
	this.jwput       = (JWPut) props.get(JWGUI.JWPUT_P);
	this.ui          = (JWGUI) props.get(JWGUI.JWUI_P);
	this.formatter   = getDateFormatter();
	jwput.addJWListener(jwlistener);
	HL hl            = new HL();
	urlhistory.addHistoryListener(hl);
	filehistory.addHistoryListener(hl);
    }

    protected void build(Hashtable props) {
	GridBagLayout gbl = new GridBagLayout();
 	GridBagConstraints gbc = new GridBagConstraints();
 	gbc.fill = GridBagConstraints.HORIZONTAL;
 	gbc.weightx = 0.9;
 	gbc.weighty = 0;
 	gbc.insets = Utilities.insets4;

	JPanel panel = new JPanel(new BorderLayout());

	//
	// source
	//
	source = new JComboBox();
	filehistory.initialize(source);
	source.setEditable(true);
	source.setMaximumRowCount(5);
	source.addItemListener(itemlistener);

	JButton browse = new JButton("Browse");
	browse.setActionCommand("select");
	browse.addActionListener(new BrowseListener(filehistory,
						    ui.getUserProperties(),
						    JWGUI.SAVE_DIR_P));

	JPanel p1 = new JPanel(new BorderLayout(4, 0));
	p1.setBorder(JWBorder.createTitledBorder("Source"));
	p1.add(source, "Center");
	p1.add(browse, "East");

	JPanel sourcep = new JPanel(gbl);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(p1, gbc);
	sourcep.add(p1);

	//
	// destination
	//
	dest = new JComboBox();
	urlhistory.initialize(dest);
	dest.setEditable(true);
	dest.setMaximumRowCount(5);

	p1 = new JPanel(new BorderLayout(4, 0));
	p1.setBorder(JWBorder.createTitledBorder("Destination"));
	p1.add(dest, "Center");

 	GridBagLayout gbl2 = new GridBagLayout();
	JPanel destp = new JPanel(gbl2);

	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbl2.setConstraints(p1, gbc);
	destp.add(p1);

	//
	// save
	//
	//JButton save = new JButton(" Upload Document ");
	JButton save = Icons.getIconButton("upload_button_normal.gif",
					   "upload_button_pressed.gif",
					   null);
	//save.setMargin(Utilities.insets5);
	ActionListener al = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		save();
	    }
	};
	save.addActionListener(al);

// 	JPanel savep = new JPanel();
// 	savep.add(save, "Center");
// 	savep.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));

	//
	// abort
	//
	//abortButton = new JButton(" Abort ");
	abortButton = Icons.getIconButton("abort_button_normal.gif",
					  "abort_button_pressed.gif",
					  "abort_button_disabled.gif");
	ActionListener al2 = new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		jwput.abort();
		setCursor(Cursor.DEFAULT_CURSOR);
		disableAbort();
	    }
	};
	abortButton.addActionListener(al2);
	abortButton.setEnabled(false);

	JPanel savep = new JPanel();
	savep.add(save, "West");
	savep.add(abortButton, "East");
	savep.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));

	//
	// source + destination + save
	//
	JPanel p3 = new JPanel(new GridLayout(2, 1));
	p3.add(sourcep);
	p3.add(destp);

	JPanel p4 = new JPanel(new BorderLayout());
	p4.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));
	p4.add(p3, "Center");

 	panel.add(p4, "North");
 	panel.add(savep, "South");

	//
	// Source informations
	//

	JLabel media = new JLabel("Media Type");
	cmedia = new JComboBox(Utils.getKnownMimeTypes());
	cmedia.setEditable(true);
	cmedia.setMaximumRowCount(5);
	cmedia.addItemListener(new MediaTypeObserver(cmedia));

	JLabel encoding = new JLabel("Content Encoding");
	cencoding = new JComboBox(encodings);
	cencoding.setEditable(true);
	cencoding.setMaximumRowCount(5);

	JLabel charset = new JLabel("Charset");
	ccharset = new JComboBox(charsets);
	ccharset.setEditable(true);
	ccharset.setMaximumRowCount(5);

	JLabel length = new JLabel("Length");
	length.setBorder(BorderFactory.createEmptyBorder(0, 20, 0, 0));
	clength = new JTextField();
	clength.setBorder(BorderFactory.createLoweredBevelBorder());
	clength.setEditable(false);

	JLabel last = new JLabel("Last Modified");
	last.setBorder(BorderFactory.createEmptyBorder(0, 20, 0, 0));
	clast = new JTextField();
	clast.setBorder(BorderFactory.createLoweredBevelBorder());
	clast.setEditable(false);

	JLabel entity = new JLabel("Entity Tag");
	entity.setBorder(BorderFactory.createEmptyBorder(0, 20, 0, 0));
	centity = new JTextField();
	centity.setBorder(BorderFactory.createLoweredBevelBorder());
	centity.setEditable(false);

	JPanel infp = new JPanel(new GridLayout(3,4,7,7));
	infp.setBorder(BorderFactory.createEmptyBorder(7, 7, 7, 7));
	infp.add(media);    infp.add(cmedia); 
	infp.add(length);   infp.add(clength);
	infp.add(encoding); infp.add(cencoding);
	infp.add(last);     infp.add(clast);
	infp.add(charset);  infp.add(ccharset);
	infp.add(entity);   infp.add(centity);

	JPanel pb2 = new JPanel(new BorderLayout());
	pb2.add(infp, "West");

	JPanel optionsp = new JPanel(new BorderLayout());
	optionsp.add(infp, "North");
	//
	// add the two panel
	//
	add("Upload Document", panel);
	add("Source Informations", optionsp);
	this.sourceInfoUpdater = new SU(optionsp);
	addChangeListener(sourceInfoUpdater);
    }

    protected void enableAbort() {
	abortButton.setEnabled(true);
    }

    protected void disableAbort() {
	abortButton.setEnabled(false);
    }

    /**
     * Get a DateFormat compliant with RFC 822 updated by RFC 1123.
     * @return a SimpleDateFormat instance.
     */
    private SimpleDateFormat getDateFormatter() {
	SimpleDateFormat formatter = 
	    new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss zzz");
	formatter.setTimeZone(TimeZone.getTimeZone("GMT"));
	return formatter;
    }

    /**
     * Get the File specified by the user
     * @return a String
     */
    protected String getFile() {
	//Don't use getSelectedItem()
	//Not always updated
	return (String)source.getEditor().getItem();
    }

    /**
     * Get the URL specified bu the user
     * @return a String
     */
    protected String getURL() {
	//Don't use getSelectedItem()
	//Not always updated
	return (String)dest.getEditor().getItem();
    }

    /**
     * Get the HTTP Headers specified by the user
     */
    protected Hashtable getHeaders() {
	Hashtable headers = new Hashtable(5);
	String mime = sourceInfoUpdater.getUserSelectedMime();
	if ((mime != null) && (! mime.equals(" "))) {
	    headers.put("Content-Type", mime);
	    String chars = (String)ccharset.getSelectedItem();
	    if ((chars != null) && (! chars.equals(" ")))
		headers.put("Content-Type", mime+=";charset="+chars);
	}
	String enc = (String)cencoding.getSelectedItem();
	if ((enc != null) && (! enc.equals(" ")))
	    headers.put("Content-Encoding", enc);
	return headers;
    }

    protected void save() {
	String url = getURL();
	if (url == null)
	    return;
	String file = getFile();
	if (file == null)
	    return;
	
	//new mime type?
	String newmime = sourceInfoUpdater.getUserSelectedMime();
	if ((newmime != null) && (! newmime.equals(" "))) {
	    if (cmedia.getSelectedIndex() == -1)
		cmedia.insertItemAt(newmime, 0);
	    //save it
	    jwput.registerMimeTypeForFile(file, newmime);
	}

	int idx = file.indexOf('|');
	if (idx != -1) {
	    //several files to put...
	    StringTokenizer st = new StringTokenizer(file, "|");
	    Vector v = new Vector();
	    while (st.hasMoreTokens()) {
		v.add(new File(st.nextToken()));
	    }
	    File files[] = new File[v.size()];
	    v.copyInto(files);

	    (new Saver(jwput, files, url, getHeaders())).start();
	} else {
	    //
	    // perform the save now
	    //
	    (new Saver(jwput, new File(file), url, getHeaders())).start();
	}
    }

    protected void setCursor(int cursor) {
	Frame frame = 
	    (Frame)SwingUtilities.getAncestorOfClass(Frame.class, this);
	frame.setCursor(Cursor.getPredefinedCursor(cursor));
    }

    /**
     * Get the JWPut tool
     */
    protected JWPut getJWPut() {
	return jwput;
    }

}
