// JWTUI.java
// $Id: JWTUI.java,v 1.13 2000/08/17 15:07:28 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html

package org.w3c.jwput.tui;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.Reader;

import org.w3c.jwput.JWUI;

/**
 * Winie's Textual User Interface
 * @version $Revision: 1.13 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class JWTUI implements JWUI{

    protected LineNumberReader in = null;

    protected String readLine() 
	throws IOException
    {
	if (in == null)
	    in = new LineNumberReader(new InputStreamReader(System.in));
	return in.readLine();
    }

    /**
     * Display an error message
     * @param msg the error message
     */
    public void displayError(String msg) {
	System.err.println(msg);
    }

    /**
     * Display an error message
     * @param msg the error message
     * @param title the title
     */
    public void displayError(String msg, String title) {
	System.err.println(msg);
    }

    /**
     * Display a error message (on several lines)
     * @param msg the message
     * @param title the title
     */
    public void displayError(String msg[], String title) {
	for (int i = 0 ; i < msg.length ; i++)
	    System.err.println(msg[i]);
    }

   /**
     * Display a message
     * @param msg the message
     * @param title the title
     */
    public void displayMessage(String msg) {
	System.out.println(msg);
    }

    /**
     * Display a message
     * @param msg the message
     * @param title the dialog title
     */
    public void displayMessage(String msg, String title) {
	System.out.println(msg);
    }

    /**
     * Display a message (on several lines)
     * @param msg the message
     * @param title the dialog title
     */
    public void displayMessage(String msg[], String title) {
	for (int i = 0 ; i < msg.length ; i++)
	    System.out.println(msg[i]);
    }

    /**
     * Log a string
     * @param the String to log
     */
    public void log(String msg) {
	System.out.println(msg);
    }

    /**
     * Log an error
     * @param the String to log
     */
    public void errlog(String msg) {
	System.err.println(msg);
    }

    public void setStatus(String stat) {
	System.out.println(stat);
    }

    /**
     * Get a File from the user.
     * @param message the message to display
     * @return a String instance
     */
    public String getFile(String title, String message) {
	return getString(message);
    }

    /**
     * Get a File from the user.
     * @param message the message to display
     * @return a String instance
     */
    public String getFile(String title, String message[]) {
	return getString(message);
    }

    /**
     * Get a String from the user.
     * @param message the message to display
     * @return a String instance
     */
    public String getString(String message) {
	System.out.print(message);
	try {
	    return readLine();
	} catch (IOException ex) {
	    return null;
	}
    }

    /**
     * Get a String from the user.
     * @param message the message to display
     * @return a String instance
     */
    public String getString(String message[]) {
	for (int i = 0 ; i < message.length ; i++)
	    System.out.println(message[i]);
	try {
	    return readLine();
	} catch (IOException ex) {
	    return null;
	}
    }

    /**
     * Ask a question to the user.
     * @param question the Question
     * return a boolean (true = yes, false = no)
     */
    public boolean askQuestion(String question) {
	System.out.print(question+" (yes/no) ");
	try {
	    return (! readLine().equalsIgnoreCase("no"));
	} catch (IOException ex) {
	    return false;
	}
    }

    /**
     * Ask a question to the user.
     * @param message a message to display before the question
     * @param question the Question
     * return a boolean (true = yes, false = no)
     */
    public boolean askQuestion(String message, String question) {
	System.out.println(message);
	return askQuestion(question);
    }

    /**
     * Ask a question to the user.
     * @param question the Question
     * @param message a message to display before the question (several lines)
     * return a boolean (true = yes, false = no)
     */
    public boolean askQuestion(String message[], String question) {
	for (int i = 0 ; i < message.length ; i++)
	    System.out.println(message[i]);
	return askQuestion(question);
    }

    /**
     * Ask a question to the user.
     * @param question the Question (several lines)
     * @param message a message to display before the question (several lines)
     * return a boolean (true = yes, false = no)
     */
    public boolean askQuestion(String message[], String question[]) {
	for (int i = 0 ; i < message.length ; i++)
	    System.out.println(message[i]);
	int idx = 0;
	for (idx = 0 ; idx < question.length-1 ; idx++) {
	    System.out.println(question[idx]);
	}
	return askQuestion(question[idx+1]);
    }

    /**
     * Ask a question to the user.
     * @param question the Question (several lines)
     * @param message a message to display before the question (several lines)
     * @return APPROVE_OPTION, CANCEL_OPTION, ABORT_OPTION
     */
    public int ask(String message[], String question[]) {
	return (askQuestion(message, question) 
		? APPROVE_OPTION : CANCEL_OPTION);
    }

    /**
     * Get a Stream on the log.
     * @return an OutputStream
     */
    public OutputStream getLogStream() {
	return System.out;
    }

    /**
     * Get a Stream on the errlog.
     * @return an OutputStream
     */
    public OutputStream getErrlogStream() {
	return System.err;
    }

    /**
     * Get a monitored inputstream
     * @param the inputstream to monitor
     * @return the monitored inputstream
     */
    public InputStream getMonitoredInputStream(InputStream in, 
					       String message,
					       int length) 
    {
	ByteCountInputStream min = new ByteCountInputStream(in, 1024);
	StreamMonitor monitor = new StreamMonitor(length, message);
	min.registerByteCountListener(monitor);
	return min;
    }

    public void sync() {
	//nothing to do;
    }

    /**
     * [0] username
     * [1] password
     */
    public String[] getAuthentication(String realm, String message,
				      boolean firstime) {
	String user     = null;
	String password = null;
	if (firstime)
	    System.err.println(message);
	else
	    System.err.println("Invalid username/password");
	if (realm != null) {
	    System.out.println("Enter username for " + realm + ';');
	}
	System.out.print("user: ");
	try {
	    user = readLine();
	    System.out.print("password: ");
	    password = readLine();
	} catch (IOException ex) {
	    System.err.println(ex.getMessage());
	    return null;
	}
	String array[] = { realm, user, password };
	return array;
    }

    /**
     * Show the log to the User (and still write in the log stream)
     */
    public void showlogs() {
    }

    /**
     * Hide the log to the User (and still write in the log stream)
     */
    public void hidelogs() {
    }    
    /**
     * No message popup
     */
    public void hidePopups() {
    }

    /**
     * Display message popups
     */
    public void showPopups() {
    }
}
